//  Copyright 2020 SILINES . Released under GPLv3 in Russia.
//---------------------------------------------------------------------------------------------------
//  Пример программы работы с устройством RODOS-4. Версия 1.02
//---------------------------------------------------------------------------------------------------
//  www.silines.ru
//---------------------------------------------------------------------------------------------------
#define R4_VID 0x20A0
#define R4_PID 0x4173
//---------------------------------------------------------------------------------------------------
#include <iostream>
#include <stdio.h>
#include <cstring>
#include <unistd.h>
#include <getopt.h>
#include <string>
#include <memory>
#include <map>
#include <utility>
#include <algorithm>
#include "RODOS4.h"
using namespace std;
//---------------------------------------------------------------------------------------------------
    
struct flags_t
{
    bool no_flags;
    bool all;
    unsigned int device_id;
    
    RODOS4Cnannel R4Device[RODOS4_MAX_CHANNEL_NUM];
    void init (void);
};
//---------------------------------------------------------------------------------------------------
int main(int argc, char* argv[])
{
    std::ios::sync_with_stdio(false);

    int MAIN_RESULT = 1;
    flags_t flags;
    flags.init();

    //  Разбор переданных параметров
    const char* short_options = "hi:a";

        const struct option long_options[] = {
            {"help",no_argument,NULL,'h'},
            {"id",required_argument,NULL,'i'},
            {"all",no_argument,NULL,'a'},
            {"c0",required_argument,NULL,'0'},
            {"c1",required_argument,NULL,'1'},
            {"c2",required_argument,NULL,'2'},
            {"c3",required_argument,NULL,'3'},
            {"c4",required_argument,NULL,'4'},
            {"c5",required_argument,NULL,'5'},
            {"c6",required_argument,NULL,'6'},
            {"c7",required_argument,NULL,'7'},
            {"c8",required_argument,NULL,'8'},
            {"c9",required_argument,NULL,'9'},
            {"c10",required_argument,NULL,'A'},
            {"c11",required_argument,NULL,'B'},
            {"c12",required_argument,NULL,'C'},
            {"c13",required_argument,NULL,'D'},
            {"c14",required_argument,NULL,'E'},
            {"c15",required_argument,NULL,'F'},
            {NULL,0,NULL,0}
        };

        int rez;
        int option_index;

        while ((rez=getopt_long(argc,argv,short_options,long_options,&option_index))!=-1)
        {
            switch(rez){
                case 'h': {
                    cout << " Используйте параметры\n"
                            "   --help,  -h    - вызов справки\n"
                            "   --id ID, -i ID - обращение к RODOS-4 с соответствующим ID\n"
                            "   --all,   -a    - для выполнения на всех RODOS-4\n"
                            "   --cX PWM       - установить ШИМ PWM[0...128] на X[0...15] выводе\n"
                            "                    для устройств с реле PWM=128 - включить реле; PWM=0 - выключить реле\n"
                            "   Запуск без параметров - получения списка всех подключенных устройств семейства RODOS-4 и их ID\n"
                         << flush;
                    flags.no_flags = false;
                    break;
                }
                case 'i': {
                    try { flags.device_id = stol(optarg); }
                    catch (...) { cout <<  "Ошибка: некорректное значение ID" << endl; return -1; }
                    flags.no_flags = false;
                    break;
                }
                case 'a': {
                    flags.all = true;
                    flags.no_flags = false;
                    break;
                }
                case '0':
                case '1':
                case '2':
                case '3':
                case '4':
                case '5':
                case '6':
                case '7':
                case '8':
                case '9':
                {
                    try {
                    flags.no_flags = false;
                    flags.R4Device[rez - '0'].ChannelChange = true;
                    flags.R4Device[rez - '0'].ChannelValue = (uint8_t)stoi(optarg);
                    if ( flags.R4Device[rez - '0'].ChannelValue > 128 ) { cout << "Ошибка: некорректное значение выхода" << endl; return -1; }
                    }
                    catch (...) { cout << "Ошибка: некорректное значение выхода" << endl; return -1; }
                    break;
                }
                case 'A':
                case 'B':
                case 'C':
                case 'D':
                case 'E':
                case 'F':
                {
                    flags.no_flags = false;
                    try {
                    flags.no_flags = false;
                    flags.R4Device[rez - 'A' + 10].ChannelChange = true;
                    flags.R4Device[rez - 'A' + 10].ChannelValue = (uint8_t)stoi(optarg);
                    }
                    catch (...) { cout << "Ошибка: некорректное значение выхода" << endl; return -1; }
                    break;
                }
                case '?': default: {
                    cout << "Неизвестный параметр" << endl;
                    break;
                }
            }
        }

    if ((flags.all)&&(flags.device_id)) { cout << "Неверно заданы параметры" << endl; MAIN_RESULT = -1; }
    else
    {
        //  Подключение к RODOS-4 и исполнение команд
        struct hid_device_info *devs;
        int device_counter = 0;

        devs = hid_enumerate(R4_VID, R4_PID);

        if (flags.no_flags) cout << "Поиск устройств..." << endl;
        
        RODOS4_t *R4 = NULL;
        while (devs)
        {
            try { R4 = new RODOS4_t(devs->path); }
            catch (...) { cout << "Memory allocation failure!" << endl; }

            if (R4->CHECK_HANDLE() == false) cout << "Ошибка подключения" << endl;
            else
            {
                wstring PrString;
                R4->Read_Device_Production_String(PrString);

                if (flags.no_flags) {
                    wcout << PrString << flush;
                    cout << " ID: " << R4->Get_Device_ID() << endl;
                }
                if ( (PrString.size() >= 7) && (!wcscmp(PrString.substr(0, 7).c_str(), L"RODOS-4")) )
                {
                    device_counter++;
                    if (((flags.device_id == 0)&&(flags.all)) || (flags.device_id == R4->Get_Device_ID()))
                    {
                        cout << "ID: " << R4->Get_Device_ID() << " " << std::string(PrString.begin(), PrString.end()) << ":" << endl;
                        for (uint8_t ChCounter=0; ChCounter < RODOS4_MAX_CHANNEL_NUM; ChCounter++ )
                            {
                             if ( flags.R4Device[ChCounter].ChannelChange )
                                {
                                 if ( R4->SET_CHANNEL_PWM(ChCounter, flags.R4Device[ChCounter].ChannelValue) ) { cout << "CH" <<to_string(ChCounter) << ": OK" << endl; }
                                 else                                                                          { cout << "CH" <<to_string(ChCounter) << ": ERR" << endl; }
                                }
                            }
                    }
                }
            }
            devs = devs->next;
            delete R4;
            R4 = NULL;
        }
        if (flags.no_flags) cout << "Найдено RODOS-4: " << device_counter << endl;
    }
    return MAIN_RESULT;
}
//---------------------------------------------------------------------------------------------------
void flags_t::init(void)
{
    no_flags = true;
    all = false;
    device_id = 0;
    for (uint8_t I=0; I<RODOS4_MAX_CHANNEL_NUM; I++)
	{
	 R4Device[I].ChannelChange = false;
	 R4Device[I].ChannelValue = 0;
	}
}
//---------------------------------------------------------------------------------------------------
